/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.examples.onlineSim.resilience;

import com.net2plan.interfaces.networkDesign.NetPlan;
import com.net2plan.interfaces.simulation.IEventProcessor;
import com.net2plan.interfaces.simulation.SimEvent;
import com.net2plan.libraries.IPUtils;
import com.net2plan.utils.Triple;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * This algorithm reacts to any resilience event (failure/restoration) trying to 
 * routing all the traffic using OSPF/ECMP with a given link IGP weight setting. These 
 * weights are not modified by the algorithm.
 * 
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @version 1.1, May 2015
 */
public class NRSim_AA_OSPF_pathRerouting extends IEventProcessor
{
	private Set<Long> linkIds;
	private Map<Long, Double> linkIGPWeightSetting;
	
	@Override
	public String getDescription()
	{
		return "This algorithm reacts to any resilience event (failure/restoration) "
			+ "trying to routing all the traffic using OSPF/ECMP with a given link IGP "
			+ "weight setting. These weights are not modified by the algorithm";
	}

	@Override
	public List<Triple<String, String, String>> getParameters()
	{
		return null;
	}

	@Override
	public void initialize(NetPlan initialNetPlan, NetPlan currentNetPlan, Map<String, String> algorithmParameters, Map<String, String> simulationParameters, Map<String, String> net2planParameters)
	{
		linkIds = currentNetPlan.getLinkIds();
		linkIGPWeightSetting = IPUtils.getLinkWeightMap(currentNetPlan);
	}

	@Override
	public void processEvent(NetPlan currentNetPlan, SimEvent event)
	{
		/* Update the IGP OSPF weight setting (set to Double.MAX_VALUE for non-working links) according to the current network state */
		Map<Long, Double> current_linkIGPWeightSetting = new LinkedHashMap<Long, Double>();
		for(long linkId : linkIds)
		{
			long originNodeId = currentNetPlan.getLinkOriginNode(linkId);
			long destinationNodeId = currentNetPlan.getLinkDestinationNode(linkId);

			if (currentNetPlan.isLinkDown(linkId) || currentNetPlan.isNodeDown(originNodeId) || currentNetPlan.isNodeDown(destinationNodeId))
				current_linkIGPWeightSetting.put(linkId, Double.MAX_VALUE);
			else
				current_linkIGPWeightSetting.put(linkId, linkIGPWeightSetting.get(linkId));
		}
		
		/* Compute and apply new routing according to current state */
		IPUtils.setECMPForwardingRulesFromLinkWeights(currentNetPlan, current_linkIGPWeightSetting);
	}
}
